# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
from six import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String


class CfxBioradBase(Splitted):

    SEPARATOR = ";"


class CfxBioradResult(object):
    FIELDS = [
        String("well"),
        String("fluor"),
        String("target"),
        String("content"),
        String("sample"),
        String("cq"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            setattr(self, field.name, line[index])


class CfxBioradResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: CfxBioradResult
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(','))
        for line in reader:
            if not len(line):
                continue
            if len(line) == 6:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {
        "code": "cfx_biorad",
        "label": "Cfx Bio-Rad CSV"
    }

    def parse(self, data):
        obj = CfxBioradResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.well == "Base Serial Number":
                instrument_serial_number = m.fluor
                continue
            elif not m.target and not m.sample and not m.cq:
                continue
            elif m.sample == "Sample" and m.well == "Well" and m.cq == "Cq":
                continue
            sample = worklist.get_or_create(
                sample_id=m.sample,
                rack="CFX-{}".format(m.well),
                type="patient"
            )
            sample.results.add(
                code=m.target,
                value=m.cq,
                instrument=instrument_serial_number
            )
            sample.results.add(
                code="{}{}".format(m.fluor, m.target),
                value=m.cq,
                instrument=instrument_serial_number
            )
        return worklist
